window.Gleam.initSearch([{"doc":"decepticon","title":"decepticon","content":"# Decepticon - StateT/Result in Gleam\n\nThis library sort of presumes familiarity with `StateT` as it exists in languages\nlike Haskell and PureScript. If you are familiar with `StateT`, then great. \n\n[Otherwise I would recommend you try out Act](https://github.com/MystPi/act).\n\nWith Decepticon we can create &quot;stateful&quot; actions that succeed or fail.\n```gleam\nimport gleeunit/should\nimport decepticon.{do, error, eval, get, put}\n\npub fn increment_succeed() {\n  use count &lt;- do(get())\n  put(count + 1)\n}\n\npub fn increment_fail() {\n  use count &lt;- do(get())\n  use _ &lt;- do(error(&quot;Oh no!&quot;))\n  put(count + 1)\n}\n```\n\nThen we can chain these actions together in ways that can succeed\n```gleam\npub fn succeed_test() {\n  let run_state = {\n    use _ &lt;- do(increment_succeed())\n    use _ &lt;- do(increment_succeed())\n    use _ &lt;- do(increment_succeed())\n    get()\n  }\n\n  let result = eval(run_state, 0)\n\n  should.equal(result, Ok(3))\n}\n```\n\nor fail:\n```gleam\npub fn fail_test() {\n  let run_state = {\n    use _ &lt;- do(increment_succeed())\n    use _ &lt;- do(increment_fail())\n    use _ &lt;- do(increment_succeed())\n    get()\n  }\n\n  let result = eval(run_state, 0)\n\n  should.be_error(result)\n}\n```\n","url":"index.html"},{"doc":"decepticon/stateful","title":"State","content":"pub type State(a, s) {\n  State(run: fn(s) -&gt; #(a, s))\n}\n\nState(run: fn(s) -&gt; #(a, s))\n\n\nSynonyms:\ndecepticon/stateful.State\ndecepticon/stateful State","url":"decepticon/stateful.html#State"},{"doc":"decepticon/stateful","title":"action","content":"pub fn action(action_value: a) -&gt; State(a, b)\n\nSynonyms:\ndecepticon/stateful.action\ndecepticon/stateful action","url":"decepticon/stateful.html#action"},{"doc":"decepticon/stateful","title":"apply","content":"pub fn apply(\n  prev: State(fn(a) -&gt; b, c),\n  next: State(a, c),\n) -&gt; State(b, c)\n\nSynonyms:\ndecepticon/stateful.apply\ndecepticon/stateful apply","url":"decepticon/stateful.html#apply"},{"doc":"decepticon/stateful","title":"do","content":"pub fn do(\n  over state: State(a, b),\n  with and_then_fn: fn(a) -&gt; State(c, b),\n) -&gt; State(c, b)\n\nSynonyms:\ndecepticon/stateful.do\ndecepticon/stateful do","url":"decepticon/stateful.html#do"},{"doc":"decepticon/stateful","title":"eval","content":"pub fn eval(state: State(a, b), initial: b) -&gt; a\n\nSynonyms:\ndecepticon/stateful.eval\ndecepticon/stateful eval","url":"decepticon/stateful.html#eval"},{"doc":"decepticon/stateful","title":"exec","content":"pub fn exec(state: State(a, b), initial: b) -&gt; b\n\nSynonyms:\ndecepticon/stateful.exec\ndecepticon/stateful exec","url":"decepticon/stateful.html#exec"},{"doc":"decepticon/stateful","title":"get","content":"pub fn get() -&gt; State(a, a)\n\nSynonyms:\ndecepticon/stateful.get\ndecepticon/stateful get","url":"decepticon/stateful.html#get"},{"doc":"decepticon/stateful","title":"increment_state","content":"pub fn increment_state() -&gt; State(Nil, Int)\n\nSynonyms:\ndecepticon/stateful.increment_state\ndecepticon/stateful increment_state","url":"decepticon/stateful.html#increment_state"},{"doc":"decepticon/stateful","title":"map","content":"pub fn map(\n  over state: State(a, b),\n  with map_fn: fn(a) -&gt; c,\n) -&gt; State(c, b)\n\nSynonyms:\ndecepticon/stateful.map\ndecepticon/stateful map","url":"decepticon/stateful.html#map"},{"doc":"decepticon/stateful","title":"put","content":"pub fn put(state_value: a) -&gt; State(Nil, a)\n\nSynonyms:\ndecepticon/stateful.put\ndecepticon/stateful put","url":"decepticon/stateful.html#put"},{"doc":"decepticon/stateful","title":"run","content":"pub fn run(state: State(a, b), initial: b) -&gt; #(a, b)\n\nSynonyms:\ndecepticon/stateful.run\ndecepticon/stateful run","url":"decepticon/stateful.html#run"},{"doc":"decepticon/stateful","title":"decepticon/stateful","content":"","url":"decepticon/stateful.html"},{"doc":"decepticon","title":"StateResult","content":"pub type StateResult(a, s, err) {\n  StateResult(run: State(Result(a, err), s))\n}\n The main type of this library- which represents a `State` wrapping a `Result`.\n See the `decepticon/stateful` module for standalone `State`\nStateResult(run: State(Result(a, err), s))\n\n\nSynonyms:\ndecepticon.StateResult\ndecepticon StateResult","url":"decepticon.html#StateResult"},{"doc":"decepticon","title":"action","content":"pub fn action(action_value: a) -&gt; StateResult(a, b, c)\n Lift a `Result`&#39;s value argument into a `StateResult`\nSynonyms:\ndecepticon.action\ndecepticon action","url":"decepticon.html#action"},{"doc":"decepticon","title":"apply","content":"pub fn apply(\n  prev: StateResult(fn(a) -&gt; b, c, d),\n  next: StateResult(a, c, d),\n) -&gt; StateResult(b, c, d)\n Applicative Functor method for `StateResult`\nSynonyms:\ndecepticon.apply\ndecepticon apply","url":"decepticon.html#apply"},{"doc":"decepticon","title":"do","content":"pub fn do(\n  over state_result: StateResult(a, b, c),\n  with and_then_fn: fn(a) -&gt; StateResult(d, b, c),\n) -&gt; StateResult(d, b, c)\n The basic mechanism of chaining `StateResult` actions. This is most similar to `result.try`\nSynonyms:\ndecepticon.do\ndecepticon do","url":"decepticon.html#do"},{"doc":"decepticon","title":"error","content":"pub fn error(error: a) -&gt; StateResult(b, c, a)\n Lift a `Result`&#39;s error argument into a `StateResult`\nSynonyms:\ndecepticon.error\ndecepticon error","url":"decepticon.html#error"},{"doc":"decepticon","title":"error_state","content":"pub fn error_state(state: State(a, b)) -&gt; StateResult(c, b, a)\n Lift a `State` type into a `StateResult` type. Wrapping it&#39;s\n action type in `result.Error`\nSynonyms:\ndecepticon.error_state\ndecepticon error_state","url":"decepticon.html#error_state"},{"doc":"decepticon","title":"eval","content":"pub fn eval(\n  state_result: StateResult(a, b, c),\n  s: b,\n) -&gt; Result(a, c)\n Given a `StateResult`, return the `Result` of the last executed action.\nSynonyms:\ndecepticon.eval\ndecepticon eval","url":"decepticon.html#eval"},{"doc":"decepticon","title":"exec","content":"pub fn exec(state_result: StateResult(a, b, c), s: b) -&gt; b\n Given a `StateResult`, return the final version of the internal state.\nSynonyms:\ndecepticon.exec\ndecepticon exec","url":"decepticon.html#exec"},{"doc":"decepticon","title":"get","content":"pub fn get() -&gt; StateResult(a, a, b)\n Return the current value of state\nSynonyms:\ndecepticon.get\ndecepticon get","url":"decepticon.html#get"},{"doc":"decepticon","title":"map","content":"pub fn map(\n  over state_result: StateResult(a, b, c),\n  with map_fn: fn(a) -&gt; d,\n) -&gt; StateResult(d, b, c)\n Map over the action type of a `StateResult`\nSynonyms:\ndecepticon.map\ndecepticon map","url":"decepticon.html#map"},{"doc":"decepticon","title":"map_error","content":"pub fn map_error(\n  over state_result: StateResult(a, b, c),\n  with map_error_fn: fn(c) -&gt; d,\n) -&gt; StateResult(a, b, d)\n Map over the error type of a `StateResult`\nSynonyms:\ndecepticon.map_error\ndecepticon map_error","url":"decepticon.html#map_error"},{"doc":"decepticon","title":"ok_state","content":"pub fn ok_state(state: State(a, b)) -&gt; StateResult(a, b, c)\n Lift a `State` type into a `StateResult` type. Wrapping it&#39;s\n action type in `result.Ok`\nSynonyms:\ndecepticon.ok_state\ndecepticon ok_state","url":"decepticon.html#ok_state"},{"doc":"decepticon","title":"put","content":"pub fn put(state_value: a) -&gt; StateResult(Nil, a, b)\n Set the current value of state\nSynonyms:\ndecepticon.put\ndecepticon put","url":"decepticon.html#put"},{"doc":"decepticon","title":"try","content":"pub fn try(result_value: Result(a, b)) -&gt; StateResult(a, c, b)\n Lift a `Result` into the `StateResult` type. Useful for when you have a\n non-stateful action that is simply a `Result` and wish to weave it in \n as a `use` statement\nSynonyms:\ndecepticon.try\ndecepticon try","url":"decepticon.html#try"},{"doc":"decepticon","title":"decepticon","content":"","url":"decepticon.html"}]);